#
#           アルゴリズム5.1.1によるシミュレーション
#
import matplotlib.pyplot as plt
import numpy as np
import scipy.stats as ss

NSamples = 10_000
L_MNs = [5, 10, 20, 50, 100, 200, 500]
L_deltas = [0.2, 0.5, 0.8]
v_alpha = 0.05
powers = np.empty((len(L_deltas), len(L_MNs)))
for i_delta, v_delta in enumerate(L_deltas):
    for j_M, M in enumerate(L_MNs):
        N = M
        df = M + N - 2
        t_a = ss.t.ppf(0.95, df)                   #   限界値
        count, i = 0, 1                            #   初期化
        while True:
            v1 = ss.norm.rvs(loc = v_delta * ((M * N) / (M + N)) ** 0.5, scale = 1)
            v2 = ss.chi2.rvs(df)
            t_v = v1 / ((v2 / df) ** 0.5)
            if t_a < t_v:                          #   有意？
                count += 1
            i += 1
            if i > NSamples:                       #   設定条件でのシミュレーション終了？
                break
        print('i_delta =', i_delta, '   delta =', v_delta, '   j_M =', j_M, '  M = N =', M)
        print('Power =', count / NSamples)
        powers[i_delta][j_M] = count / NSamples    #   検定力のシミュレーション推定値

print('Powers =\n', powers)
#
#         シミュレーション結果の描画
#
v_log_x = np.log(L_MNs)
L_log_x = ['{}'.format(v) for v in L_MNs]
for j in range(len(L_deltas)):
    plt.plot(v_log_x, powers[j], label = '$\delta$ = {}'.format(L_deltas[j]))
plt.xticks(v_log_x, L_log_x)
plt.xlabel('M = N', fontsize = 16)
plt.yticks([0, 0.5, 0.95, 1])
plt.ylabel('Power', fontsize = 16)
plt.hlines(1.0, np.log(5), np.log(500), lw = 1)
plt.hlines(0.95, np.log(5), np.log(500), lw = 1)
plt.hlines(0.5, np.log(5), np.log(500), lw = 1)
plt.hlines(0, np.log(5), np.log(500), lw = 1)
plt.legend(loc = 'lower right')
plt.title('Effect Size $\delta$, Sample Size M, and Power', fontsize = 18)
plt.tight_layout()
plt.savefig('Fig_delta.png')
plt.show()
